@@ -1,27 +1,35 @@
-## -*- coding: utf-8 -*-
-##
-## batch.py
-##
-## Author:   Toke Høiland-Jørgensen (toke@toke.dk)
-## Date:     11 April 2014
-## Copyright (c) 2014-2015, Toke Høiland-Jørgensen
-##
-## This program is free software: you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation, either version 3 of the License, or
-## (at your option) any later version.
-##
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# -*- coding: utf-8 -*-
+#
+# batch.py
+#
+# Author:   Toke Høiland-Jørgensen (toke@toke.dk)
+# Date:     11 April 2014
+# Copyright (c) 2014-2016, Toke Høiland-Jørgensen
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-import sys, pprint, string, re, time, os, subprocess, signal, itertools, traceback, io, uuid
+import itertools
+import os
+import pprint
+import re
+import signal
+import subprocess
+import sys
+import time
+import uuid
 
 from datetime import datetime, timedelta
 from fnmatch import fnmatch
@@ -32,26 +40,28 @@ try:
 except ImportError:
     from ConfigParser import RawConfigParser
 
-from flent import aggregators, formatters, resultset
-from flent.metadata import record_extended_metadata, record_postrun_metadata
-from flent.util import clean_path, path_components, format_date
-from flent.settings import CONFIG_TYPES
+from flent import aggregators, formatters, resultset, loggers
+from flent.metadata import record_metadata, record_postrun_metadata
+from flent.util import clean_path, format_date
 
 # Python2/3 compatibility
 try:
     basestring
 except NameError:
-    basestring=str
+    basestring = str
+
+logger = loggers.get_logger(__name__)
+
 
 def new(settings):
     return BatchRunner(settings)
 
+
 class BatchRunner(object):
 
-    _INTERP_REGEX =  re.compile(r"(^|[^$])(\$\{([^}]+)\})")
+    _INTERP_REGEX = re.compile(r"(^|[^$])(\$\{([^}]+)\})")
     _MAX_INTERP = 1000
 
-
     def __init__(self, settings):
         self.args = OrderedDict()
         self.batches = OrderedDict()
@@ -59,13 +69,12 @@ class BatchRunner(object):
         self.settings = settings
         self.killed = False
         self.children = []
-        self.log_fd = None
+        self.logfile = self.logfile_debug = None
         self.tests_run = 0
 
         for f in settings.BATCH_FILES:
             self.read(f)
 
-
     def read(self, filename):
         parser = RawConfigParser(dict_type=OrderedDict)
         read = parser.read(filename)
@@ -73,7 +82,7 @@ class BatchRunner(object):
             raise RuntimeError("Unable to read batch file: %s." % filename)
 
         for s in parser.sections():
-            typ,nam = s.split("::")
+            typ, nam = s.split("::")
             if typ.lower() == 'arg':
                 self.args[nam.lower()] = OrderedDict(parser.items(s))
             elif typ.lower() == 'batch':
@@ -91,14 +100,16 @@ class BatchRunner(object):
                 # Expand inheritance
                 if 'inherits' in vals:
                     if not vals['inherits'] in obj:
-                        raise RuntimeError("%s inherits from non-existent parent %s." % (name, vals['inherits']))
+                        raise RuntimeError(
+                            "%s inherits from non-existent parent %s."
+                            % (name, vals['inherits']))
                     obj[name] = self.inherit(obj[vals['inherits']], vals)
 
                 # Parse boolean options
-                for k,v in obj[name].items():
-                    if isinstance(v, basestring) and v.lower() in ('yes', 'true', 'on'):
+                for k, v in obj[name].items():
+                    if isinstance(v, basestring) and v.lower() in ('yes', 'true', 'on'):  # noqa: E501
                         obj[name][k] = True
-                    elif isinstance(v, basestring) and v.lower() in ('no', 'false', 'off'):
+                    elif isinstance(v, basestring) and v.lower() in ('no', 'false', 'off'):  # noqa: E501
                         obj[name][k] = False
 
     def inherit(self, parent, child):
@@ -146,20 +157,22 @@ class BatchRunner(object):
             m = self._INTERP_REGEX.search(ret)
             i += 1
             if i > self._MAX_INTERP:
-                raise RuntimeError("Cyclic interpolation (more than %d expansions performed)." % self._MAX_INTERP)
+                raise RuntimeError(
+                    "Cyclic interpolation (more than %d expansions performed)."
+                    % self._MAX_INTERP)
 
         return ret.replace("$$", "$")
 
     def apply_args(self, values, args={}, settings=None):
         new = OrderedDict(args)
         new.update(values)
-        for k,v in new.items():
+        for k, v in new.items():
             new[k] = self.interpolate(v, new, settings)
 
         return new
 
     def commands_for(self, batch, settings=None):
-        if not 'commands' in batch:
+        if 'commands' not in batch:
             return []
         cmdnames = [i.strip() for i in batch['commands'].split(',')]
         commands = OrderedDict()
@@ -168,8 +181,9 @@ class BatchRunner(object):
             c = cmdnames.pop(0)
             if not c or c in commands:
                 continue
-            if not c in self.commands:
-                raise RuntimeError("Can't find command '%s' when expanding batch command." % c)
+            if c not in self.commands:
+                raise RuntimeError("Can't find command '%s' when expanding "
+                                   "batch command." % c)
             cmd = self.apply_args(self.commands[c], batch, settings)
 
             # Don't include disabled commands
@@ -178,7 +192,8 @@ class BatchRunner(object):
 
             # Commands can specify extra commands to run; expand those, use the
             # dictionary to prevent duplicates
-            extra = [i.strip() for i in cmd.get('extra_commands', '').split(',') if i.strip()]
+            extra = [i.strip() for i in cmd.get('extra_commands', '').split(',')
+                     if i.strip()]
             for e in reversed(extra):
                 cmdnames.insert(0, e)
             commands[c] = cmd
@@ -188,37 +203,41 @@ class BatchRunner(object):
     def run_command(self, command):
         cmd = command['exec'].strip()
         if self.settings.BATCH_VERBOSE:
+            ess = command.get('essential', False)
             if self.settings.BATCH_DRY:
-                sys.stderr.write("  Would run '%s'" % cmd)
+                logger.info("  Would run '%s' (%s)", cmd,
+                            'essential' if ess else 'non-essential')
             else:
-                sys.stderr.write("  Running '%s'" % cmd)
-            if command.get('essential', False):
-                sys.stderr.write(" (essential).\n")
-            else:
-                sys.stderr.write(" (non-essential).\n")
+                logger.info("  Running '%s', (%s)", cmd,
+                            'essential' if ess else 'non-essential')
 
         if self.settings.BATCH_DRY:
             return
         if command['type'] in ('pre', 'post'):
             try:
-                res = subprocess.check_output(cmd, universal_newlines=True, shell=True,
+                res = subprocess.check_output(cmd, universal_newlines=True,
+                                              shell=True,
                                               stderr=subprocess.STDOUT)
-                self.log("%s: %s" % (cmd, res))
+                logger.debug("Command '%s' executed successfully.", cmd,
+                             extra={'output': res})
             except subprocess.CalledProcessError as e:
                 if command.get('essential', False):
                     raise RuntimeError("Essential command '%s' failed. "
-                                       "Return code: %s.\nOutput:\n %s." % (cmd, e.returncode,
-                                                                            "\n ".join(e.output.splitlines())))
+                                       "Return code: %s.\nOutput:\n %s."
+                                       % (cmd, e.returncode,
+                                          "\n ".join(e.output.splitlines())))
                 else:
-                    self.log("%s err(%d): %s" % (cmd, e.returncode,
-                                                  "\n ".join(e.output.splitlines())))
+                    logger.warning("Command '%s' failed (%d).",
+                                   cmd,
+                                   e.returncode,
+                                   extra={'output': e.output})
         elif command['type'] in ('monitor',):
             proc = subprocess.Popen(cmd, universal_newlines=True, shell=True,
-                                              stderr=subprocess.STDOUT)
-            self.children.append((proc,command.get('kill', False)))
+                                    stderr=subprocess.STDOUT)
+            self.children.append((proc, command.get('kill', False)))
 
     def kill_children(self, force=False):
-        for proc,kill in self.children:
+        for proc, kill in self.children:
             if kill or force:
                 proc.terminate()
             else:
@@ -227,8 +246,8 @@ class BatchRunner(object):
 
     def run_commands(self, commands, ctype, essential_only=False):
         for c in commands:
-            if c['type'] == ctype and (not essential_only
-                                       or c.get('essential', False)):
+            if c['type'] == ctype and (not essential_only or
+                                       c.get('essential', False)):
                 try:
                     self.run_command(c)
                 except:
@@ -237,43 +256,41 @@ class BatchRunner(object):
                     if not essential_only:
                         raise
 
-
     def gen_filename(self, settings, batch, argset, rep):
         filename = "batch-%s-%s-%s" % (
             settings.BATCH_NAME,
             batch['batch_time'],
             batch.get('filename_extra', "%s-%s" % (argset, rep))
-            )
+        )
         return clean_path(filename)
 
     def expand_argsets(self, batch, argsets, batch_time, batch_name,
-                       print_status = True):
+                       print_status=True):
 
         for argset in itertools.product(*argsets):
             rep = argset[-1]
             argset = argset[:-1]
             settings = self.settings.copy()
             if print_status:
-                sys.stderr.write(" args:%s rep:%02d" % (",".join(argset),rep))
-                if settings.BATCH_DRY:
-                    sys.stderr.write(" (dry run)")
-                sys.stderr.write(".\n")
+                logger.info(" args:%s rep:%02d%s", ",".join(argset), rep,
+                            " (dry run)" if settings.BATCH_DRY else "")
             settings.FORMAT = 'null'
             settings.BATCH_NAME = batch_name
             settings.BATCH_TIME = batch_time
             settings.TIME = datetime.utcnow()
 
             expand_vars = {'repetition': "%02d" % rep,
-                           'batch_time': format_date(settings.BATCH_TIME, fmt="%Y-%m-%dT%H%M%S")}
+                           'batch_time': format_date(settings.BATCH_TIME,
+                                                     fmt="%Y-%m-%dT%H%M%S")}
 
             for arg in argset:
-                if not arg in self.args:
+                if arg not in self.args:
                     raise RuntimeError("Invalid arg: '%s'." % arg)
                 expand_vars.update(self.args[arg])
 
             b = self.apply_args(batch, expand_vars, settings)
 
-            if not 'test_name' in b:
+            if 'test_name' not in b:
                 raise RuntimeError("Missing test name.")
 
             settings.load_rcvalues(b.items(), override=True)
@@ -291,49 +308,49 @@ class BatchRunner(object):
                 argset = []
                 for a in batch[k].split(','):
                     a = a.strip().lower()
-                    matches = [arg for arg in self.args if fnmatch(arg,a)]
+                    matches = [arg for arg in self.args if fnmatch(arg, a)]
                     if not matches:
                         raise RuntimeError("No matches for arg: '%s'." % a)
                     argset.extend(matches)
                 argsets.append(argset)
 
-        reps = range(1,int(batch.get('repetitions', 1))+1)
+        reps = range(1, int(batch.get('repetitions', 1)) + 1)
         argsets.append(reps)
 
         return argsets
 
     def get_batch_runtime(self, batch_name):
-        if not batch_name in self.batches:
+        if batch_name not in self.batches:
             raise RuntimeError("Can't find batch '%s' to run." % batch_name)
         batch = self.batches[batch_name].copy()
         batch.update(self.settings.BATCH_OVERRIDE)
 
         if batch.get('abstract', False) or batch.get('disabled', False):
-            return (0,0)
+            return (0, 0)
 
         total_time = 0
         n = 0
         argsets = self.get_argsets(batch)
 
-        for _,s in self.expand_argsets(batch, argsets, self.settings.TIME, batch_name, False):
+        for _, s in self.expand_argsets(batch, argsets, self.settings.TIME,
+                                        batch_name, False):
             total_time += s.TOTAL_LENGTH + int(batch.get('pause', 0))
             n += 1
 
         return total_time, n
 
-
     def run_batch(self, batch_name):
-        if not batch_name in self.batches:
+        if batch_name not in self.batches:
             raise RuntimeError("Can't find batch '%s' to run." % batch_name)
         batch = self.batches[batch_name].copy()
         batch.update(self.settings.BATCH_OVERRIDE)
 
         # A batch declared 'abstract' is not runnable
         if batch.get('abstract', False):
-            sys.stderr.write(" Batch marked as abstract. Not running.\n")
+            logger.info(" Batch marked as abstract. Not running.")
             return False
         elif batch.get('disabled', False) or not batch.get('enabled', True):
-            sys.stderr.write(" Batch disabled.\n")
+            logger.info(" Batch disabled.")
             return False
 
         argsets = self.get_argsets(batch)
@@ -341,10 +358,12 @@ class BatchRunner(object):
         pause = int(batch.get('pause', 0))
 
         batch_time = None
-        if self.settings.BATCH_RESUME is not None and os.path.isdir(self.settings.BATCH_RESUME):
+        if self.settings.BATCH_RESUME is not None and \
+           os.path.isdir(self.settings.BATCH_RESUME):
             # We're resuming a batch run. Try to find a data file we can get the
             # original batch run time from.
-            for dirpath, dirnames, filenames in os.walk(self.settings.BATCH_RESUME):
+            for dirpath, dirnames, filenames in os.walk(
+                    self.settings.BATCH_RESUME):
                 datafiles = [f for f in filenames if f.endswith(resultset.SUFFIX)]
                 if datafiles:
                     f = datafiles[0]
@@ -352,20 +371,24 @@ class BatchRunner(object):
                     batch_time = r.meta("BATCH_TIME")
                     try:
                         self.settings.BATCH_UUID = r.meta("BATCH_UUID")
-                        sys.stderr.write(" Using previous UUID %s.\n" % self.settings.BATCH_UUID)
+                        logger.info(" Using previous UUID %s.\n",
+                                    self.settings.BATCH_UUID)
                     except KeyError:
                         pass
                     break
             if batch_time is None:
-                raise RuntimeError("No data files found in resume directory %s." % self.settings.BATCH_RESUME)
+                raise RuntimeError("No data files found in resume directory %s."
+                                   % self.settings.BATCH_RESUME)
         elif self.settings.BATCH_RESUME:
-            raise RuntimeError("Batch resume directory %s doesn't exist!\n" % self.settings.BATCH_RESUME)
+            raise RuntimeError("Batch resume directory %s doesn't exist!\n"
+                               % self.settings.BATCH_RESUME)
         else:
             batch_time = self.settings.TIME
 
         filenames_seen = set()
 
-        for b,settings in self.expand_argsets(batch, argsets, batch_time, batch_name):
+        for b, settings in self.expand_argsets(batch, argsets,
+                                               batch_time, batch_name):
 
             if 'output_path' in b:
                 output_path = clean_path(b['output_path'], allow_dirs=True)
@@ -373,29 +396,43 @@ class BatchRunner(object):
                 output_path = settings.DATA_DIR
 
             if settings.BATCH_RESUME is not None:
-                if os.path.commonprefix([os.path.abspath(output_path),
-                                         os.path.abspath(settings.BATCH_RESUME)]) != os.path.abspath(settings.BATCH_RESUME):
-                    raise RuntimeError("Batch-specified output path is not a subdirectory of resume path. Bailing.")
-                if os.path.exists(os.path.join(output_path, "%s%s" % (settings.DATA_FILENAME, resultset.SUFFIX))):
-                    sys.stderr.write("  Previous result exists, skipping.\n")
+                if os.path.commonprefix(
+                    [os.path.abspath(output_path),
+                     os.path.abspath(settings.BATCH_RESUME)]) \
+                        != os.path.abspath(settings.BATCH_RESUME):
+                    raise RuntimeError("Batch-specified output path is not a "
+                                       "subdirectory of resume path. Bailing.")
+                if os.path.exists(os.path.join(output_path, "%s%s"
+                                               % (settings.DATA_FILENAME,
+                                                  resultset.SUFFIX))):
+                    logger.info("  Previous result exists, skipping.")
                     continue
 
             if settings.BATCH_DRY and settings.BATCH_VERBOSE:
-                sys.stderr.write("  Would output to: %s.\n" % output_path)
+                logger.info("  Would output to: %s.\n", output_path)
             elif not settings.BATCH_DRY and not os.path.exists(output_path):
                 try:
                     os.makedirs(output_path)
                 except OSError as e:
-                    raise RuntimeError("Unable to create output path '%s': %s." % (output_path,e))
+                    raise RuntimeError("Unable to create output path '%s': %s."
+                                       % (output_path, e))
 
             commands = self.commands_for(b, settings)
             if not settings.BATCH_DRY:
-                self.log_fd = io.open(os.path.join(output_path,"%s.log" % settings.DATA_FILENAME), "at")
+                self.logfile = loggers.setup_logfile(
+                    os.path.join(output_path,
+                                 "%s.log" % settings.DATA_FILENAME),
+                    level=loggers.INFO)
             if b.get('debug_log', False):
-                settings.LOG_FILE = os.path.join(output_path,"%s.debug.log" % settings.DATA_FILENAME)
+                self.logfile_debug = loggers.setup_logfile(
+                    os.path.join(output_path,
+                                 "%s.debug.log" % settings.DATA_FILENAME),
+                    level=loggers.DEBUG,
+                    maxlevel=loggers.DEBUG)
 
             if settings.DATA_FILENAME in filenames_seen:
-                sys.stderr.write("  Warning: Filename already seen in this run: %s\n" % settings.DATA_FILENAME)
+                logger.warning("Filename already seen in this run: %s",
+                               settings.DATA_FILENAME)
             filenames_seen.add(settings.DATA_FILENAME)
 
             self.run_commands(commands, 'pre')
@@ -403,13 +440,13 @@ class BatchRunner(object):
             try:
                 if settings.BATCH_VERBOSE:
                     if settings.BATCH_DRY:
-                        sys.stderr.write("  Would run test '%s'.\n" % settings.NAME)
+                        logger.info("  Would run test '%s'.", settings.NAME)
                     else:
-                        sys.stderr.write("  Running test '%s'.\n" % settings.NAME)
-                    sys.stderr.write("   data_filename=%s\n" % settings.DATA_FILENAME)
-                    for k in sorted([i.lower() for i in CONFIG_TYPES.keys()]):
-                        if k in b:
-                            sys.stderr.write("   %s=%s\n" % (k, b[k]))
+                        logger.info("  Running test '%s'.", settings.NAME)
+                    logger.info("   data_filename=%s", settings.DATA_FILENAME)
+                    for k in sorted(b.keys()):
+                        if k in settings.parser:
+                            logger.info("   %s=%s", k, b[k])
 
                 if settings.BATCH_DRY:
                     self.tests_run += 1
@@ -418,54 +455,53 @@ class BatchRunner(object):
             except KeyboardInterrupt:
                 self.run_commands(commands, 'post', essential_only=True)
                 raise
-            except:
+            except Exception as e:
                 self.run_commands(commands, 'post', essential_only=True)
-                sys.stderr.write("  Error running test: %s\n" % "  ".join(traceback.format_exception_only(*sys.exc_info()[:2])))
-                if settings.DEBUG_ERROR:
-                    traceback.print_exc()
+                logger.exception("  Error running test: %s", str(e))
             else:
                 try:
                     self.run_commands(commands, 'post')
-                except:
+                except Exception as e:
                     self.run_commands(commands, 'post', essential_only=True)
-                    sys.stderr.write("  Error running post-commands: %s\n" % "  ".join(traceback.format_exception_only(*sys.exc_info()[:2])))
-                    if settings.DEBUG_ERROR:
-                        traceback.print_exc()
+                    logger.exception("  Error running post-commands: %s", str(e))
             finally:
                 self.kill_children()
-                if self.log_fd:
-                    self.log_fd.close()
-                self.log_fd = None
+                if self.logfile:
+                    loggers.remove_log_handler(self.logfile)
+                    self.logfile.close()
+                    self.logfile = None
+                if self.logfile_debug:
+                    loggers.remove_log_handler(self.logfile_debug)
+                    self.logfile_debug.close()
+                    self.logfile_debug = None
 
             if settings.BATCH_DRY and settings.BATCH_VERBOSE:
-                sys.stderr.write("  Would sleep for %d seconds.\n" % pause)
+                logger.info("  Would sleep for %d seconds.", pause)
             elif not settings.BATCH_DRY:
                 time.sleep(pause)
 
-
-    def log(self, text):
-        if self.log_fd is not None:
-            self.log_fd.write(text + "\n")
-
     def run_test(self, settings, output_path, print_datafile_loc=False):
         settings = settings.copy()
         settings.load_test()
         res = resultset.new(settings)
-        if settings.EXTENDED_METADATA:
-            record_extended_metadata(res, settings.REMOTE_METADATA)
+        record_metadata(res, settings.EXTENDED_METADATA,
+                        settings.REMOTE_METADATA)
 
         if not settings.HOSTS:
             raise RuntimeError("Must specify host (-H option).")
 
+        logger.info("Starting %s test. Expected run time: %d seconds.",
+                    settings.NAME, settings.TOTAL_LENGTH)
+
         self.agg = aggregators.new(settings)
         res = self.agg.postprocess(self.agg.aggregate(res))
         if self.killed:
             return
-        if settings.EXTENDED_METADATA:
-            record_postrun_metadata(res, settings.REMOTE_METADATA)
+        record_postrun_metadata(res, settings.EXTENDED_METADATA,
+                                settings.REMOTE_METADATA)
         res.dump_dir(output_path)
-        if print_datafile_loc:
-            sys.stderr.write("Data file written to %s.\n" % res.dump_filename)
+        logger.log(loggers.INFO if print_datafile_loc else loggers.DEBUG,
+                   "Data file written to %s.", res.dump_filename)
 
         formatter = formatters.new(settings)
         formatter.format([res])
@@ -476,10 +512,14 @@ class BatchRunner(object):
         settings = settings.copy()
         results = []
         test_name = None
-        for i,filename in enumerate(settings.INPUT):
+        for i, filename in enumerate(settings.INPUT):
             r = resultset.load(filename, settings.ABSOLUTE_TIME)
-            if test_name is not None and test_name != r.meta("NAME") and not settings.GUI:
-                sys.stderr.write("Warning: result sets are not from the same test (found %s/%s).\n" % (test_name, r.meta("NAME")))
+            if test_name is not None and test_name != r.meta("NAME") and \
+               not settings.GUI:
+                logger.warning("Result sets are not from the same "
+                               "test (found %s/%s).",
+                               test_name,
+                               r.meta("NAME"))
             test_name = r.meta("NAME")
             if results and settings.CONCATENATE:
                 results[0].concatenate(r)
@@ -488,9 +528,6 @@ class BatchRunner(object):
                     r.set_label(settings.OVERRIDE_LABELS[i])
                 results.append(r)
 
-        if settings.GUI:
-            load_gui(settings)
-
         settings.update(results[0].meta())
         settings.load_test(informational=True)
 
@@ -502,12 +539,19 @@ class BatchRunner(object):
         formatter = formatters.new(settings)
         formatter.format(results)
 
-    def fork_and_run(self):
+    def fork_and_run(self, queue):
         pid = os.fork()
         if pid:
             return pid
         else:
-            self.run()
+            loggers.set_queue_handler(queue)
+            signal.signal(signal.SIGTERM, self.kill)
+            try:
+                self.run()
+            except Exception as e:
+                logger.exception(str(e))
+            queue.close()
+            queue.join_thread()
             os._exit(0)
 
     def run(self):
@@ -516,35 +560,45 @@ class BatchRunner(object):
         elif self.settings.BATCH_NAMES:
             start_time = self.settings.TIME
             self.settings.BATCH_UUID = str(uuid.uuid4())
-            sys.stderr.write("Started batch run %s at %s.\n" % (self.settings.BATCH_UUID, format_date(start_time, fmt="%Y-%m-%d %H:%M:%S")))
-            if len(self.settings.BATCH_NAMES) == 1 and self.settings.BATCH_NAMES[0] == 'ALL':
-                sys.stderr.write("Running all batches.\n")
+            logger.info("Started batch run %s at %s.",
+                        self.settings.BATCH_UUID,
+                        format_date(start_time, fmt="%Y-%m-%d %H:%M:%S"))
+
+            if len(self.settings.BATCH_NAMES) == 1 and \
+               self.settings.BATCH_NAMES[0] == 'ALL':
+                logger.info("Running all batches.")
                 batches = self.batches.keys()
             else:
                 batches = self.settings.BATCH_NAMES
             runtimes = [self.get_batch_runtime(b) for b in batches]
             total_time, total_n = map(sum, zip(*runtimes))
             if total_time > 0:
-                sys.stderr.write("Estimated total runtime: %s (%d tests)\n" % (timedelta(seconds=total_time), total_n))
+                logger.info("Estimated total runtime: %s (%d tests)",
+                            timedelta(seconds=total_time),
+                            total_n)
             for b in batches:
                 try:
-                    sys.stderr.write("Running batch '%s'.\n" % b)
+                    logger.info("Running batch '%s'.", b)
                     self.run_batch(b)
                 except RuntimeError:
                     raise
                 except Exception as e:
                     if self.settings.DEBUG_ERROR:
                         raise
-                    raise RuntimeError("Error while running batch '%s': %r." % (b, e))
+                    raise RuntimeError("Error while running batch '%s': %r."
+                                       % (b, e))
             end_time = datetime.utcnow()
-            sys.stderr.write("Ended batch sequence at %s. %s %d tests in %s.\n" % (format_date(end_time, fmt="%Y-%m-%d %H:%M:%S"),
-                                                                                   "Ran" if not self.settings.BATCH_DRY else 'Would have run',
-                                                                                   self.tests_run, (end_time - start_time)))
+            logger.info("Ended batch sequence at %s. %s %d tests in %s.",
+                        format_date(end_time, fmt="%Y-%m-%d %H:%M:%S"),
+                        "Ran" if not self.settings.BATCH_DRY else 'Would have run',  # noqa: E501
+                        self.tests_run,
+                        (end_time - start_time))
             return True
         else:
             return self.run_test(self.settings, self.settings.DATA_DIR, True)
 
-    def kill(self):
+    def kill(self, *args):
+        logger.debug("Killing child processes")
         self.killed = True
         self.kill_children(force=True)
         try:
@@ -554,7 +608,7 @@ class BatchRunner(object):
 
     def p(self):
         for t in 'args', 'batches', 'commands':
-            print("%s:\n%s\n"% (t, pprint.pformat(getattr(self, t))))
+            print("%s:\n%s\n" % (t, pprint.pformat(getattr(self, t))))
 
 
 if __name__ == "__main__":
